/****************************************************************************
** Meta object code from reading C++ file 'mainwindow.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "mainwindow.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'mainwindow.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MainWindow_t {
    QByteArrayData data[23];
    char stringdata0[292];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MainWindow_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MainWindow_t qt_meta_stringdata_MainWindow = {
    {
QT_MOC_LITERAL(0, 0, 10), // "MainWindow"
QT_MOC_LITERAL(1, 11, 11), // "nsmOpenFile"
QT_MOC_LITERAL(2, 23, 0), // ""
QT_MOC_LITERAL(3, 24, 4), // "name"
QT_MOC_LITERAL(4, 29, 10), // "unixSignal"
QT_MOC_LITERAL(5, 40, 2), // "fd"
QT_MOC_LITERAL(6, 43, 7), // "fileNew"
QT_MOC_LITERAL(7, 51, 8), // "fileOpen"
QT_MOC_LITERAL(8, 60, 12), // "fileOpenDemo"
QT_MOC_LITERAL(9, 73, 22), // "fileOpenDemoInstrument"
QT_MOC_LITERAL(10, 96, 8), // "fileSave"
QT_MOC_LITERAL(11, 105, 10), // "fileSaveAs"
QT_MOC_LITERAL(12, 116, 17), // "updateWindowTitle"
QT_MOC_LITERAL(13, 134, 12), // "helpAboutAms"
QT_MOC_LITERAL(14, 147, 11), // "helpAboutQt"
QT_MOC_LITERAL(15, 159, 19), // "recentFileActivated"
QT_MOC_LITERAL(16, 179, 8), // "QAction*"
QT_MOC_LITERAL(17, 188, 20), // "setupRecentFilesMenu"
QT_MOC_LITERAL(18, 209, 16), // "applyPreferences"
QT_MOC_LITERAL(19, 226, 18), // "displayPreferences"
QT_MOC_LITERAL(20, 245, 14), // "viewFullscreen"
QT_MOC_LITERAL(21, 260, 22), // "handleJackSessionEvent"
QT_MOC_LITERAL(22, 283, 8) // "openFile"

    },
    "MainWindow\0nsmOpenFile\0\0name\0unixSignal\0"
    "fd\0fileNew\0fileOpen\0fileOpenDemo\0"
    "fileOpenDemoInstrument\0fileSave\0"
    "fileSaveAs\0updateWindowTitle\0helpAboutAms\0"
    "helpAboutQt\0recentFileActivated\0"
    "QAction*\0setupRecentFilesMenu\0"
    "applyPreferences\0displayPreferences\0"
    "viewFullscreen\0handleJackSessionEvent\0"
    "openFile"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MainWindow[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      18,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    1,  104,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       4,    1,  107,    2, 0x08 /* Private */,
       6,    0,  110,    2, 0x08 /* Private */,
       7,    0,  111,    2, 0x08 /* Private */,
       8,    0,  112,    2, 0x08 /* Private */,
       9,    0,  113,    2, 0x08 /* Private */,
      10,    0,  114,    2, 0x08 /* Private */,
      11,    0,  115,    2, 0x08 /* Private */,
      12,    0,  116,    2, 0x08 /* Private */,
      13,    0,  117,    2, 0x08 /* Private */,
      14,    0,  118,    2, 0x08 /* Private */,
      15,    1,  119,    2, 0x08 /* Private */,
      17,    0,  122,    2, 0x08 /* Private */,
      18,    0,  123,    2, 0x08 /* Private */,
      19,    0,  124,    2, 0x08 /* Private */,
      20,    0,  125,    2, 0x08 /* Private */,
      21,    1,  126,    2, 0x0a /* Public */,
      22,    1,  129,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void, QMetaType::QString,    3,

 // slots: parameters
    QMetaType::Void, QMetaType::Int,    5,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 16,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    2,
    QMetaType::Void, QMetaType::QString,    2,

       0        // eod
};

void MainWindow::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MainWindow *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->nsmOpenFile((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        case 1: _t->unixSignal((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 2: _t->fileNew(); break;
        case 3: _t->fileOpen(); break;
        case 4: _t->fileOpenDemo(); break;
        case 5: _t->fileOpenDemoInstrument(); break;
        case 6: _t->fileSave(); break;
        case 7: _t->fileSaveAs(); break;
        case 8: _t->updateWindowTitle(); break;
        case 9: _t->helpAboutAms(); break;
        case 10: _t->helpAboutQt(); break;
        case 11: _t->recentFileActivated((*reinterpret_cast< QAction*(*)>(_a[1]))); break;
        case 12: _t->setupRecentFilesMenu(); break;
        case 13: _t->applyPreferences(); break;
        case 14: _t->displayPreferences(); break;
        case 15: _t->viewFullscreen(); break;
        case 16: _t->handleJackSessionEvent((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 17: _t->openFile((*reinterpret_cast< const QString(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (MainWindow::*)(const QString & );
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&MainWindow::nsmOpenFile)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MainWindow::staticMetaObject = { {
    QMetaObject::SuperData::link<QMainWindow::staticMetaObject>(),
    qt_meta_stringdata_MainWindow.data,
    qt_meta_data_MainWindow,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MainWindow::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MainWindow::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MainWindow.stringdata0))
        return static_cast<void*>(this);
    return QMainWindow::qt_metacast(_clname);
}

int MainWindow::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QMainWindow::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 18)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 18;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 18)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 18;
    }
    return _id;
}

// SIGNAL 0
void MainWindow::nsmOpenFile(const QString & _t1)
{
    void *_a[] = { nullptr, const_cast<void*>(reinterpret_cast<const void*>(std::addressof(_t1))) };
    QMetaObject::activate(this, &staticMetaObject, 0, _a);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
