/****************************************************************************
** Meta object code from reading C++ file 'prefwidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "prefwidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'prefwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_PrefWidget_t {
    QByteArrayData data[13];
    char stringdata0[226];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_PrefWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_PrefWidget_t qt_meta_stringdata_PrefWidget = {
    {
QT_MOC_LITERAL(0, 0, 10), // "PrefWidget"
QT_MOC_LITERAL(1, 11, 11), // "prefChanged"
QT_MOC_LITERAL(2, 23, 0), // ""
QT_MOC_LITERAL(3, 24, 22), // "colorBackgroundClicked"
QT_MOC_LITERAL(4, 47, 28), // "colorModuleBackgroundClicked"
QT_MOC_LITERAL(5, 76, 24), // "colorModuleBorderClicked"
QT_MOC_LITERAL(6, 101, 22), // "colorModuleFontClicked"
QT_MOC_LITERAL(7, 124, 20), // "colorPortFontClicked"
QT_MOC_LITERAL(8, 145, 17), // "colorCableClicked"
QT_MOC_LITERAL(9, 163, 19), // "defaultcolorClicked"
QT_MOC_LITERAL(10, 183, 16), // "colorJackClicked"
QT_MOC_LITERAL(11, 200, 17), // "enableGridToggled"
QT_MOC_LITERAL(12, 218, 7) // "checked"

    },
    "PrefWidget\0prefChanged\0\0colorBackgroundClicked\0"
    "colorModuleBackgroundClicked\0"
    "colorModuleBorderClicked\0"
    "colorModuleFontClicked\0colorPortFontClicked\0"
    "colorCableClicked\0defaultcolorClicked\0"
    "colorJackClicked\0enableGridToggled\0"
    "checked"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_PrefWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      10,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       1,       // signalCount

 // signals: name, argc, parameters, tag, flags
       1,    0,   64,    2, 0x06 /* Public */,

 // slots: name, argc, parameters, tag, flags
       3,    0,   65,    2, 0x0a /* Public */,
       4,    0,   66,    2, 0x0a /* Public */,
       5,    0,   67,    2, 0x0a /* Public */,
       6,    0,   68,    2, 0x0a /* Public */,
       7,    0,   69,    2, 0x0a /* Public */,
       8,    0,   70,    2, 0x0a /* Public */,
       9,    0,   71,    2, 0x0a /* Public */,
      10,    0,   72,    2, 0x0a /* Public */,
      11,    1,   73,    2, 0x0a /* Public */,

 // signals: parameters
    QMetaType::Void,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Bool,   12,

       0        // eod
};

void PrefWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<PrefWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->prefChanged(); break;
        case 1: _t->colorBackgroundClicked(); break;
        case 2: _t->colorModuleBackgroundClicked(); break;
        case 3: _t->colorModuleBorderClicked(); break;
        case 4: _t->colorModuleFontClicked(); break;
        case 5: _t->colorPortFontClicked(); break;
        case 6: _t->colorCableClicked(); break;
        case 7: _t->defaultcolorClicked(); break;
        case 8: _t->colorJackClicked(); break;
        case 9: _t->enableGridToggled((*reinterpret_cast< bool(*)>(_a[1]))); break;
        default: ;
        }
    } else if (_c == QMetaObject::IndexOfMethod) {
        int *result = reinterpret_cast<int *>(_a[0]);
        {
            using _t = void (PrefWidget::*)();
            if (*reinterpret_cast<_t *>(_a[1]) == static_cast<_t>(&PrefWidget::prefChanged)) {
                *result = 0;
                return;
            }
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject PrefWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QDialog::staticMetaObject>(),
    qt_meta_stringdata_PrefWidget.data,
    qt_meta_data_PrefWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *PrefWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PrefWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_PrefWidget.stringdata0))
        return static_cast<void*>(this);
    return QDialog::qt_metacast(_clname);
}

int PrefWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 10)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 10;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 10)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 10;
    }
    return _id;
}

// SIGNAL 0
void PrefWidget::prefChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
