/****************************************************************************
** Meta object code from reading C++ file 'modularsynth.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "modularsynth.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'modularsynth.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_ModularSynth_t {
    QByteArrayData data[98];
    char stringdata0[1221];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ModularSynth_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ModularSynth_t qt_meta_stringdata_ModularSynth = {
    {
QT_MOC_LITERAL(0, 0, 12), // "ModularSynth"
QT_MOC_LITERAL(1, 13, 21), // "displayMidiController"
QT_MOC_LITERAL(2, 35, 0), // ""
QT_MOC_LITERAL(3, 36, 20), // "displayParameterView"
QT_MOC_LITERAL(4, 57, 20), // "displayLadspaPlugins"
QT_MOC_LITERAL(5, 78, 10), // "midiAction"
QT_MOC_LITERAL(6, 89, 2), // "fd"
QT_MOC_LITERAL(7, 92, 10), // "startSynth"
QT_MOC_LITERAL(8, 103, 9), // "stopSynth"
QT_MOC_LITERAL(9, 113, 12), // "new_textEdit"
QT_MOC_LITERAL(10, 126, 9), // "newM_ad_2"
QT_MOC_LITERAL(11, 136, 9), // "newM_ad_4"
QT_MOC_LITERAL(12, 146, 9), // "newM_ad_6"
QT_MOC_LITERAL(13, 156, 11), // "newM_advenv"
QT_MOC_LITERAL(14, 168, 11), // "newM_advmcv"
QT_MOC_LITERAL(15, 180, 8), // "newM_amp"
QT_MOC_LITERAL(16, 189, 17), // "newM_analogmemory"
QT_MOC_LITERAL(17, 207, 13), // "newM_bitgrind"
QT_MOC_LITERAL(18, 221, 9), // "newM_conv"
QT_MOC_LITERAL(19, 231, 8), // "newM_cvs"
QT_MOC_LITERAL(20, 240, 10), // "newM_delay"
QT_MOC_LITERAL(21, 251, 8), // "newM_env"
QT_MOC_LITERAL(22, 260, 15), // "newM_function_1"
QT_MOC_LITERAL(23, 276, 15), // "newM_function_2"
QT_MOC_LITERAL(24, 292, 15), // "newM_function_4"
QT_MOC_LITERAL(25, 308, 15), // "newM_hysteresis"
QT_MOC_LITERAL(26, 324, 8), // "newM_inv"
QT_MOC_LITERAL(27, 333, 11), // "newM_ladspa"
QT_MOC_LITERAL(28, 345, 20), // "p_ladspaDesFuncIndex"
QT_MOC_LITERAL(29, 366, 1), // "n"
QT_MOC_LITERAL(30, 368, 15), // "p_newLadspaPoly"
QT_MOC_LITERAL(31, 384, 14), // "p_extCtrlPorts"
QT_MOC_LITERAL(32, 399, 8), // "newM_lfo"
QT_MOC_LITERAL(33, 408, 8), // "newM_mcv"
QT_MOC_LITERAL(34, 417, 10), // "newM_mix_2"
QT_MOC_LITERAL(35, 428, 10), // "newM_mix_4"
QT_MOC_LITERAL(36, 439, 10), // "newM_mix_8"
QT_MOC_LITERAL(37, 450, 11), // "newM_mphlfo"
QT_MOC_LITERAL(38, 462, 10), // "newM_noise"
QT_MOC_LITERAL(39, 473, 11), // "newM_noise2"
QT_MOC_LITERAL(40, 485, 14), // "newM_quantizer"
QT_MOC_LITERAL(41, 500, 12), // "newM_ringmod"
QT_MOC_LITERAL(42, 513, 10), // "newM_scmcv"
QT_MOC_LITERAL(43, 524, 8), // "QString*"
QT_MOC_LITERAL(44, 533, 11), // "p_scalaName"
QT_MOC_LITERAL(45, 545, 16), // "newM_scquantizer"
QT_MOC_LITERAL(46, 562, 11), // "newM_seq_12"
QT_MOC_LITERAL(47, 574, 11), // "newM_seq_16"
QT_MOC_LITERAL(48, 586, 11), // "newM_seq_24"
QT_MOC_LITERAL(49, 598, 11), // "newM_seq_32"
QT_MOC_LITERAL(50, 610, 10), // "newM_seq_8"
QT_MOC_LITERAL(51, 621, 9), // "newM_slew"
QT_MOC_LITERAL(52, 631, 16), // "newM_stereomix_2"
QT_MOC_LITERAL(53, 648, 16), // "newM_stereomix_4"
QT_MOC_LITERAL(54, 665, 16), // "newM_stereomix_8"
QT_MOC_LITERAL(55, 682, 16), // "newM_v8sequencer"
QT_MOC_LITERAL(56, 699, 12), // "newM_vca_exp"
QT_MOC_LITERAL(57, 712, 12), // "newM_vca_lin"
QT_MOC_LITERAL(58, 725, 12), // "newM_vcdelay"
QT_MOC_LITERAL(59, 738, 18), // "newM_vcdoubledecay"
QT_MOC_LITERAL(60, 757, 10), // "newM_vcenv"
QT_MOC_LITERAL(61, 768, 11), // "newM_vcenv2"
QT_MOC_LITERAL(62, 780, 8), // "newM_vcf"
QT_MOC_LITERAL(63, 789, 8), // "newM_vco"
QT_MOC_LITERAL(64, 798, 9), // "newM_vco2"
QT_MOC_LITERAL(65, 808, 14), // "newM_vcpanning"
QT_MOC_LITERAL(66, 823, 11), // "newM_vquant"
QT_MOC_LITERAL(67, 835, 12), // "newM_vocoder"
QT_MOC_LITERAL(68, 848, 19), // "newM_dynamicwaves_4"
QT_MOC_LITERAL(69, 868, 19), // "newM_dynamicwaves_6"
QT_MOC_LITERAL(70, 888, 19), // "newM_dynamicwaves_8"
QT_MOC_LITERAL(71, 908, 12), // "newM_midiout"
QT_MOC_LITERAL(72, 921, 10), // "newM_pcmin"
QT_MOC_LITERAL(73, 932, 11), // "newM_pcmout"
QT_MOC_LITERAL(74, 944, 10), // "newM_scope"
QT_MOC_LITERAL(75, 955, 7), // "newM_sh"
QT_MOC_LITERAL(76, 963, 13), // "newM_spectrum"
QT_MOC_LITERAL(77, 977, 14), // "newM_vcorgan_4"
QT_MOC_LITERAL(78, 992, 14), // "newM_vcorgan_6"
QT_MOC_LITERAL(79, 1007, 14), // "newM_vcorgan_8"
QT_MOC_LITERAL(80, 1022, 13), // "newM_vcswitch"
QT_MOC_LITERAL(81, 1036, 11), // "newM_wavout"
QT_MOC_LITERAL(82, 1048, 14), // "resizeTextEdit"
QT_MOC_LITERAL(83, 1063, 3), // "pos"
QT_MOC_LITERAL(84, 1067, 12), // "portSelected"
QT_MOC_LITERAL(85, 1080, 5), // "Port*"
QT_MOC_LITERAL(86, 1086, 12), // "deleteModule"
QT_MOC_LITERAL(87, 1099, 14), // "deleteTextEdit"
QT_MOC_LITERAL(88, 1114, 9), // "TextEdit*"
QT_MOC_LITERAL(89, 1124, 2), // "te"
QT_MOC_LITERAL(90, 1127, 4), // "load"
QT_MOC_LITERAL(91, 1132, 12), // "QTextStream&"
QT_MOC_LITERAL(92, 1145, 4), // "save"
QT_MOC_LITERAL(93, 1150, 10), // "loadColors"
QT_MOC_LITERAL(94, 1161, 10), // "saveColors"
QT_MOC_LITERAL(95, 1172, 12), // "allVoicesOff"
QT_MOC_LITERAL(96, 1185, 13), // "refreshColors"
QT_MOC_LITERAL(97, 1199, 21) // "redrawPortConnections"

    },
    "ModularSynth\0displayMidiController\0\0"
    "displayParameterView\0displayLadspaPlugins\0"
    "midiAction\0fd\0startSynth\0stopSynth\0"
    "new_textEdit\0newM_ad_2\0newM_ad_4\0"
    "newM_ad_6\0newM_advenv\0newM_advmcv\0"
    "newM_amp\0newM_analogmemory\0newM_bitgrind\0"
    "newM_conv\0newM_cvs\0newM_delay\0newM_env\0"
    "newM_function_1\0newM_function_2\0"
    "newM_function_4\0newM_hysteresis\0"
    "newM_inv\0newM_ladspa\0p_ladspaDesFuncIndex\0"
    "n\0p_newLadspaPoly\0p_extCtrlPorts\0"
    "newM_lfo\0newM_mcv\0newM_mix_2\0newM_mix_4\0"
    "newM_mix_8\0newM_mphlfo\0newM_noise\0"
    "newM_noise2\0newM_quantizer\0newM_ringmod\0"
    "newM_scmcv\0QString*\0p_scalaName\0"
    "newM_scquantizer\0newM_seq_12\0newM_seq_16\0"
    "newM_seq_24\0newM_seq_32\0newM_seq_8\0"
    "newM_slew\0newM_stereomix_2\0newM_stereomix_4\0"
    "newM_stereomix_8\0newM_v8sequencer\0"
    "newM_vca_exp\0newM_vca_lin\0newM_vcdelay\0"
    "newM_vcdoubledecay\0newM_vcenv\0newM_vcenv2\0"
    "newM_vcf\0newM_vco\0newM_vco2\0newM_vcpanning\0"
    "newM_vquant\0newM_vocoder\0newM_dynamicwaves_4\0"
    "newM_dynamicwaves_6\0newM_dynamicwaves_8\0"
    "newM_midiout\0newM_pcmin\0newM_pcmout\0"
    "newM_scope\0newM_sh\0newM_spectrum\0"
    "newM_vcorgan_4\0newM_vcorgan_6\0"
    "newM_vcorgan_8\0newM_vcswitch\0newM_wavout\0"
    "resizeTextEdit\0pos\0portSelected\0Port*\0"
    "deleteModule\0deleteTextEdit\0TextEdit*\0"
    "te\0load\0QTextStream&\0save\0loadColors\0"
    "saveColors\0allVoicesOff\0refreshColors\0"
    "redrawPortConnections"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ModularSynth[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      87,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,  449,    2, 0x0a /* Public */,
       3,    0,  450,    2, 0x0a /* Public */,
       4,    0,  451,    2, 0x0a /* Public */,
       5,    1,  452,    2, 0x0a /* Public */,
       7,    0,  455,    2, 0x0a /* Public */,
       8,    0,  456,    2, 0x0a /* Public */,
       9,    0,  457,    2, 0x0a /* Public */,
      10,    0,  458,    2, 0x0a /* Public */,
      11,    0,  459,    2, 0x0a /* Public */,
      12,    0,  460,    2, 0x0a /* Public */,
      13,    0,  461,    2, 0x0a /* Public */,
      14,    0,  462,    2, 0x0a /* Public */,
      15,    0,  463,    2, 0x0a /* Public */,
      16,    0,  464,    2, 0x0a /* Public */,
      17,    0,  465,    2, 0x0a /* Public */,
      18,    0,  466,    2, 0x0a /* Public */,
      19,    0,  467,    2, 0x0a /* Public */,
      20,    0,  468,    2, 0x0a /* Public */,
      21,    0,  469,    2, 0x0a /* Public */,
      22,    0,  470,    2, 0x0a /* Public */,
      23,    0,  471,    2, 0x0a /* Public */,
      24,    0,  472,    2, 0x0a /* Public */,
      25,    0,  473,    2, 0x0a /* Public */,
      26,    0,  474,    2, 0x0a /* Public */,
      27,    4,  475,    2, 0x0a /* Public */,
      32,    0,  484,    2, 0x0a /* Public */,
      33,    0,  485,    2, 0x0a /* Public */,
      34,    0,  486,    2, 0x0a /* Public */,
      35,    0,  487,    2, 0x0a /* Public */,
      36,    0,  488,    2, 0x0a /* Public */,
      37,    0,  489,    2, 0x0a /* Public */,
      38,    0,  490,    2, 0x0a /* Public */,
      39,    0,  491,    2, 0x0a /* Public */,
      40,    0,  492,    2, 0x0a /* Public */,
      41,    0,  493,    2, 0x0a /* Public */,
      42,    0,  494,    2, 0x0a /* Public */,
      42,    1,  495,    2, 0x0a /* Public */,
      45,    0,  498,    2, 0x0a /* Public */,
      45,    1,  499,    2, 0x0a /* Public */,
      46,    0,  502,    2, 0x0a /* Public */,
      47,    0,  503,    2, 0x0a /* Public */,
      48,    0,  504,    2, 0x0a /* Public */,
      49,    0,  505,    2, 0x0a /* Public */,
      50,    0,  506,    2, 0x0a /* Public */,
      51,    0,  507,    2, 0x0a /* Public */,
      52,    0,  508,    2, 0x0a /* Public */,
      53,    0,  509,    2, 0x0a /* Public */,
      54,    0,  510,    2, 0x0a /* Public */,
      55,    0,  511,    2, 0x0a /* Public */,
      56,    0,  512,    2, 0x0a /* Public */,
      57,    0,  513,    2, 0x0a /* Public */,
      58,    0,  514,    2, 0x0a /* Public */,
      59,    0,  515,    2, 0x0a /* Public */,
      60,    0,  516,    2, 0x0a /* Public */,
      61,    0,  517,    2, 0x0a /* Public */,
      62,    0,  518,    2, 0x0a /* Public */,
      63,    0,  519,    2, 0x0a /* Public */,
      64,    0,  520,    2, 0x0a /* Public */,
      65,    0,  521,    2, 0x0a /* Public */,
      66,    0,  522,    2, 0x0a /* Public */,
      67,    0,  523,    2, 0x0a /* Public */,
      68,    0,  524,    2, 0x0a /* Public */,
      69,    0,  525,    2, 0x0a /* Public */,
      70,    0,  526,    2, 0x0a /* Public */,
      71,    0,  527,    2, 0x0a /* Public */,
      72,    0,  528,    2, 0x0a /* Public */,
      73,    0,  529,    2, 0x0a /* Public */,
      74,    0,  530,    2, 0x0a /* Public */,
      75,    0,  531,    2, 0x0a /* Public */,
      76,    0,  532,    2, 0x0a /* Public */,
      77,    0,  533,    2, 0x0a /* Public */,
      78,    0,  534,    2, 0x0a /* Public */,
      79,    0,  535,    2, 0x0a /* Public */,
      80,    0,  536,    2, 0x0a /* Public */,
      81,    0,  537,    2, 0x0a /* Public */,
      82,    1,  538,    2, 0x0a /* Public */,
      84,    1,  541,    2, 0x0a /* Public */,
      86,    0,  544,    2, 0x0a /* Public */,
      87,    0,  545,    2, 0x0a /* Public */,
      87,    1,  546,    2, 0x0a /* Public */,
      90,    1,  549,    2, 0x0a /* Public */,
      92,    1,  552,    2, 0x0a /* Public */,
      93,    0,  555,    2, 0x0a /* Public */,
      94,    0,  556,    2, 0x0a /* Public */,
      95,    0,  557,    2, 0x0a /* Public */,
      96,    0,  558,    2, 0x0a /* Public */,
      97,    0,  559,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,    6,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int, QMetaType::Int, QMetaType::Bool, QMetaType::Bool,   28,   29,   30,   31,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 43,   44,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 43,   44,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::QPoint,   83,
    QMetaType::Void, 0x80000000 | 85,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 88,   89,
    QMetaType::Void, 0x80000000 | 91,    2,
    QMetaType::Void, 0x80000000 | 91,    2,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,

       0        // eod
};

void ModularSynth::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<ModularSynth *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->displayMidiController(); break;
        case 1: _t->displayParameterView(); break;
        case 2: _t->displayLadspaPlugins(); break;
        case 3: _t->midiAction((*reinterpret_cast< int(*)>(_a[1]))); break;
        case 4: _t->startSynth(); break;
        case 5: _t->stopSynth(); break;
        case 6: _t->new_textEdit(); break;
        case 7: _t->newM_ad_2(); break;
        case 8: _t->newM_ad_4(); break;
        case 9: _t->newM_ad_6(); break;
        case 10: _t->newM_advenv(); break;
        case 11: _t->newM_advmcv(); break;
        case 12: _t->newM_amp(); break;
        case 13: _t->newM_analogmemory(); break;
        case 14: _t->newM_bitgrind(); break;
        case 15: _t->newM_conv(); break;
        case 16: _t->newM_cvs(); break;
        case 17: _t->newM_delay(); break;
        case 18: _t->newM_env(); break;
        case 19: _t->newM_function_1(); break;
        case 20: _t->newM_function_2(); break;
        case 21: _t->newM_function_4(); break;
        case 22: _t->newM_hysteresis(); break;
        case 23: _t->newM_inv(); break;
        case 24: _t->newM_ladspa((*reinterpret_cast< int(*)>(_a[1])),(*reinterpret_cast< int(*)>(_a[2])),(*reinterpret_cast< bool(*)>(_a[3])),(*reinterpret_cast< bool(*)>(_a[4]))); break;
        case 25: _t->newM_lfo(); break;
        case 26: _t->newM_mcv(); break;
        case 27: _t->newM_mix_2(); break;
        case 28: _t->newM_mix_4(); break;
        case 29: _t->newM_mix_8(); break;
        case 30: _t->newM_mphlfo(); break;
        case 31: _t->newM_noise(); break;
        case 32: _t->newM_noise2(); break;
        case 33: _t->newM_quantizer(); break;
        case 34: _t->newM_ringmod(); break;
        case 35: _t->newM_scmcv(); break;
        case 36: _t->newM_scmcv((*reinterpret_cast< QString*(*)>(_a[1]))); break;
        case 37: _t->newM_scquantizer(); break;
        case 38: _t->newM_scquantizer((*reinterpret_cast< QString*(*)>(_a[1]))); break;
        case 39: _t->newM_seq_12(); break;
        case 40: _t->newM_seq_16(); break;
        case 41: _t->newM_seq_24(); break;
        case 42: _t->newM_seq_32(); break;
        case 43: _t->newM_seq_8(); break;
        case 44: _t->newM_slew(); break;
        case 45: _t->newM_stereomix_2(); break;
        case 46: _t->newM_stereomix_4(); break;
        case 47: _t->newM_stereomix_8(); break;
        case 48: _t->newM_v8sequencer(); break;
        case 49: _t->newM_vca_exp(); break;
        case 50: _t->newM_vca_lin(); break;
        case 51: _t->newM_vcdelay(); break;
        case 52: _t->newM_vcdoubledecay(); break;
        case 53: _t->newM_vcenv(); break;
        case 54: _t->newM_vcenv2(); break;
        case 55: _t->newM_vcf(); break;
        case 56: _t->newM_vco(); break;
        case 57: _t->newM_vco2(); break;
        case 58: _t->newM_vcpanning(); break;
        case 59: _t->newM_vquant(); break;
        case 60: _t->newM_vocoder(); break;
        case 61: _t->newM_dynamicwaves_4(); break;
        case 62: _t->newM_dynamicwaves_6(); break;
        case 63: _t->newM_dynamicwaves_8(); break;
        case 64: _t->newM_midiout(); break;
        case 65: _t->newM_pcmin(); break;
        case 66: _t->newM_pcmout(); break;
        case 67: _t->newM_scope(); break;
        case 68: _t->newM_sh(); break;
        case 69: _t->newM_spectrum(); break;
        case 70: _t->newM_vcorgan_4(); break;
        case 71: _t->newM_vcorgan_6(); break;
        case 72: _t->newM_vcorgan_8(); break;
        case 73: _t->newM_vcswitch(); break;
        case 74: _t->newM_wavout(); break;
        case 75: _t->resizeTextEdit((*reinterpret_cast< const QPoint(*)>(_a[1]))); break;
        case 76: _t->portSelected((*reinterpret_cast< Port*(*)>(_a[1]))); break;
        case 77: _t->deleteModule(); break;
        case 78: _t->deleteTextEdit(); break;
        case 79: _t->deleteTextEdit((*reinterpret_cast< TextEdit*(*)>(_a[1]))); break;
        case 80: _t->load((*reinterpret_cast< QTextStream(*)>(_a[1]))); break;
        case 81: _t->save((*reinterpret_cast< QTextStream(*)>(_a[1]))); break;
        case 82: _t->loadColors(); break;
        case 83: _t->saveColors(); break;
        case 84: _t->allVoicesOff(); break;
        case 85: _t->refreshColors(); break;
        case 86: _t->redrawPortConnections(); break;
        default: ;
        }
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<int*>(_a[0]) = -1; break;
        case 76:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<int*>(_a[0]) = -1; break;
            case 0:
                *reinterpret_cast<int*>(_a[0]) = qRegisterMetaType< Port* >(); break;
            }
            break;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject ModularSynth::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_ModularSynth.data,
    qt_meta_data_ModularSynth,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ModularSynth::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ModularSynth::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ModularSynth.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int ModularSynth::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 87)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 87;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 87)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 87;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
