/****************************************************************************
** Meta object code from reading C++ file 'midiwidget.h'
**
** Created by: The Qt Meta Object Compiler version 67 (Qt 5.15.19)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include <memory>
#include "midiwidget.h"
#include <QtCore/qbytearray.h>
#include <QtCore/qmetatype.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'midiwidget.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 67
#error "This file was generated using the moc from 5.15.19. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

QT_BEGIN_MOC_NAMESPACE
QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
struct qt_meta_stringdata_MidiControllerModel_t {
    QByteArrayData data[1];
    char stringdata0[20];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MidiControllerModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MidiControllerModel_t qt_meta_stringdata_MidiControllerModel = {
    {
QT_MOC_LITERAL(0, 0, 19) // "MidiControllerModel"

    },
    "MidiControllerModel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MidiControllerModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void MidiControllerModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject MidiControllerModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_meta_stringdata_MidiControllerModel.data,
    qt_meta_data_MidiControllerModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MidiControllerModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MidiControllerModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MidiControllerModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int MidiControllerModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_ModuleModel_t {
    QByteArrayData data[1];
    char stringdata0[12];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_ModuleModel_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_ModuleModel_t qt_meta_stringdata_ModuleModel = {
    {
QT_MOC_LITERAL(0, 0, 11) // "ModuleModel"

    },
    "ModuleModel"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_ModuleModel[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
       0,    0, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

       0        // eod
};

void ModuleModel::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    (void)_o;
    (void)_id;
    (void)_c;
    (void)_a;
}

QT_INIT_METAOBJECT const QMetaObject ModuleModel::staticMetaObject = { {
    QMetaObject::SuperData::link<QAbstractItemModel::staticMetaObject>(),
    qt_meta_stringdata_ModuleModel.data,
    qt_meta_data_ModuleModel,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *ModuleModel::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *ModuleModel::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_ModuleModel.stringdata0))
        return static_cast<void*>(this);
    return QAbstractItemModel::qt_metacast(_clname);
}

int ModuleModel::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QAbstractItemModel::qt_metacall(_c, _id, _a);
    return _id;
}
struct qt_meta_stringdata_MidiWidget_t {
    QByteArrayData data[19];
    char stringdata0[241];
};
#define QT_MOC_LITERAL(idx, ofs, len) \
    Q_STATIC_BYTE_ARRAY_DATA_HEADER_INITIALIZER_WITH_OFFSET(len, \
    qptrdiff(offsetof(qt_meta_stringdata_MidiWidget_t, stringdata0) + ofs \
        - idx * sizeof(QByteArrayData)) \
    )
static const qt_meta_stringdata_MidiWidget_t qt_meta_stringdata_MidiWidget = {
    {
QT_MOC_LITERAL(0, 0, 10), // "MidiWidget"
QT_MOC_LITERAL(1, 11, 15), // "clearAllClicked"
QT_MOC_LITERAL(2, 27, 0), // ""
QT_MOC_LITERAL(3, 28, 12), // "clearClicked"
QT_MOC_LITERAL(4, 41, 11), // "bindClicked"
QT_MOC_LITERAL(5, 53, 25), // "addToParameterViewClicked"
QT_MOC_LITERAL(6, 79, 14), // "toggleMidiSign"
QT_MOC_LITERAL(7, 94, 17), // "guiControlChanged"
QT_MOC_LITERAL(8, 112, 14), // "QItemSelection"
QT_MOC_LITERAL(9, 127, 8), // "selected"
QT_MOC_LITERAL(10, 136, 10), // "deselected"
QT_MOC_LITERAL(11, 147, 18), // "midiControlChanged"
QT_MOC_LITERAL(12, 166, 10), // "setLogMode"
QT_MOC_LITERAL(13, 177, 2), // "on"
QT_MOC_LITERAL(14, 180, 9), // "setNewMin"
QT_MOC_LITERAL(15, 190, 9), // "setNewMax"
QT_MOC_LITERAL(16, 200, 16), // "setInitialMinMax"
QT_MOC_LITERAL(17, 217, 17), // "updateMidiChannel"
QT_MOC_LITERAL(18, 235, 5) // "index"

    },
    "MidiWidget\0clearAllClicked\0\0clearClicked\0"
    "bindClicked\0addToParameterViewClicked\0"
    "toggleMidiSign\0guiControlChanged\0"
    "QItemSelection\0selected\0deselected\0"
    "midiControlChanged\0setLogMode\0on\0"
    "setNewMin\0setNewMax\0setInitialMinMax\0"
    "updateMidiChannel\0index"
};
#undef QT_MOC_LITERAL

static const uint qt_meta_data_MidiWidget[] = {

 // content:
       8,       // revision
       0,       // classname
       0,    0, // classinfo
      12,   14, // methods
       0,    0, // properties
       0,    0, // enums/sets
       0,    0, // constructors
       0,       // flags
       0,       // signalCount

 // slots: name, argc, parameters, tag, flags
       1,    0,   74,    2, 0x0a /* Public */,
       3,    0,   75,    2, 0x0a /* Public */,
       4,    0,   76,    2, 0x0a /* Public */,
       5,    0,   77,    2, 0x0a /* Public */,
       6,    0,   78,    2, 0x0a /* Public */,
       7,    2,   79,    2, 0x0a /* Public */,
      11,    2,   84,    2, 0x0a /* Public */,
      12,    1,   89,    2, 0x0a /* Public */,
      14,    0,   92,    2, 0x0a /* Public */,
      15,    0,   93,    2, 0x0a /* Public */,
      16,    0,   94,    2, 0x0a /* Public */,
      17,    1,   95,    2, 0x0a /* Public */,

 // slots: parameters
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, 0x80000000 | 8, 0x80000000 | 8,    9,   10,
    QMetaType::Void, 0x80000000 | 8, 0x80000000 | 8,    9,   10,
    QMetaType::Void, QMetaType::Bool,   13,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void,
    QMetaType::Void, QMetaType::Int,   18,

       0        // eod
};

void MidiWidget::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    if (_c == QMetaObject::InvokeMetaMethod) {
        auto *_t = static_cast<MidiWidget *>(_o);
        (void)_t;
        switch (_id) {
        case 0: _t->clearAllClicked(); break;
        case 1: _t->clearClicked(); break;
        case 2: _t->bindClicked(); break;
        case 3: _t->addToParameterViewClicked(); break;
        case 4: _t->toggleMidiSign(); break;
        case 5: _t->guiControlChanged((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        case 6: _t->midiControlChanged((*reinterpret_cast< const QItemSelection(*)>(_a[1])),(*reinterpret_cast< const QItemSelection(*)>(_a[2]))); break;
        case 7: _t->setLogMode((*reinterpret_cast< bool(*)>(_a[1]))); break;
        case 8: _t->setNewMin(); break;
        case 9: _t->setNewMax(); break;
        case 10: _t->setInitialMinMax(); break;
        case 11: _t->updateMidiChannel((*reinterpret_cast< int(*)>(_a[1]))); break;
        default: ;
        }
    }
}

QT_INIT_METAOBJECT const QMetaObject MidiWidget::staticMetaObject = { {
    QMetaObject::SuperData::link<QWidget::staticMetaObject>(),
    qt_meta_stringdata_MidiWidget.data,
    qt_meta_data_MidiWidget,
    qt_static_metacall,
    nullptr,
    nullptr
} };


const QMetaObject *MidiWidget::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *MidiWidget::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_meta_stringdata_MidiWidget.stringdata0))
        return static_cast<void*>(this);
    return QWidget::qt_metacast(_clname);
}

int MidiWidget::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = QWidget::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 12)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 12;
    } else if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 12)
            *reinterpret_cast<int*>(_a[0]) = -1;
        _id -= 12;
    }
    return _id;
}
QT_WARNING_POP
QT_END_MOC_NAMESPACE
